@@ -1,4 +1,6 @@
+import itertools
 import logging.config
+import math
 import re
 import socket
 import time
@@ -10,11 +12,12 @@ import networkx as nx
 import numpy as np
 import overpy
 import pyproj
+from heapdict import heapdict
 from overpy import Result
 from tqdm.auto import tqdm
 
 from pyridy.osm.utils import QueryResult, OSMLevelCrossing, OSMRailwaySwitch, OSMRailwaySignal, OSMRailwayLine, \
-    OSMRailwayElement, OSMRailwayMilestone
+    OSMRailwayElement, OSMRailwayMilestone, calc_angle_between
 from pyridy.utils.tools import internet
 
 logger = logging.getLogger(__name__)
@@ -81,7 +84,7 @@ class OSM:
         if download:
             self.download_track_data(recurse=recurse)
 
-            # Add nodes
+            # Add nodes to Graph
             self.G.add_nodes_from([(n.id, n.__dict__) for n in self.nodes])
 
             # Add edges, use node distances as weight
@@ -90,6 +93,8 @@ class OSM:
                          for n1, n2 in zip(w.nodes, w.nodes[1:])]
                 self.G.add_weighted_edges_from(edges, weight="d", way_id=w.id)
 
+            self._check_allowed_switch_transits()
+
         logger.info("Initialized region: %f, %f (SW), %f, %f (NE)" % (self.lon_sw,
                                                                       self.lat_sw,
                                                                       self.lon_ne,
@@ -121,6 +126,36 @@ class OSM:
 
         return track_query, route_query
 
+    def _check_allowed_switch_transits(self):
+        """ Checks in what ways a switch can be transited, i.e. what combination of neighbouring nodes are allowed
+
+        """
+        if not len(self.G.nodes):
+            raise ValueError("Can't determine allowed switch transits if Graph G has no nodes")
+
+        for sw in self.get_switches():
+            sw_x, sw_y = self.utm_proj(sw.lon, sw.lat)
+            sw_nbs = list(self.G.adj[sw.id])
+
+            allowed_transits = []
+            for n1, n2 in itertools.product(sw_nbs, repeat=2):
+                if n1 == n2:
+                    continue
+                else:
+                    n1_x, n1_y = self.G.nodes[n1]['attributes'].get('x', 0), self.G.nodes[n1]['attributes'].get('y', 0)
+                    n2_x, n2_y = self.G.nodes[n2]['attributes'].get('x', 0), self.G.nodes[n2]['attributes'].get('y', 0)
+
+                    v1 = [n1_x - sw_x, n1_y - sw_y]
+                    v2 = [n2_x - sw_x, n2_y - sw_y]
+
+                    ang = calc_angle_between(v1, v2)
+                    if ang > math.pi/2:
+                        allowed_transits.append((n1, sw.id, n2))
+
+            sw.allowed_transits = allowed_transits
+            self.G.nodes[sw.id]['attributes']['allowed_transits'] = allowed_transits
+        pass
+
     def download_track_data(self, railway_type: Union[List, str] = None, recurse: str = ">"):
         if recurse not in [">", ">>", "<", "<<"]:
             raise ValueError("recurse type %s not supported" % recurse)
@@ -230,6 +265,12 @@ class OSM:
             return None
 
     def get_all_route_nodes(self) -> list:
+        """ Retrieves a list of nodes part of any relation/route
+
+        Returns
+        -------
+        List[overpy.node]
+        """
         nodes = []
 
         for railway_type in self.desired_railway_types:
@@ -237,6 +278,106 @@ class OSM:
 
         return list(chain.from_iterable(nodes))
 
+    def get_shortest_path(self, source: int, target: int, weight: str, method="dijkstra") -> List[int]:
+        """ Calculates the shortest path between a source and target node. Also considers how switches can be transited
+        Based on: https://en.wikipedia.org/wiki/Dijkstra
+
+        Parameters
+        ----------
+        source: int
+            ID of source node
+        target: int
+            ID of target node
+        weight: str
+            Weight to be used for shortest path calculation, e.g. the length of the edges
+        method: str
+            Can be 'dijkstra' or 'A*'
+
+        Returns
+        -------
+        List[int]
+            List of node ids that represent the shortest path between source and target
+        """
+        dist = {n: np.inf for n in self.G.nodes}
+        prev = {n: None for n in self.G.nodes}
+
+        if method == "dijkstra":
+            dist[source] = 0
+
+            Q = heapdict()
+            for v in self.G.nodes:
+                Q[v] = dist[v]
+
+            while Q:
+                u = Q.popitem()[0]
+
+                if u == target:
+                    break
+
+                u_is_switch = True if self.G.nodes[u]['tags'].get('railway') == 'switch' else False
+
+                for v in set(self.G.adj[u]).intersection(set(Q.keys())):  # Neighbors of u that are still in Q
+                    if u_is_switch:
+                        u_prev = prev[u]
+                        allowed = True if (u_prev, u, v) in self.G.nodes[u]['attributes'].get('allowed_transits',
+                                                                                              []) else False
+                    else:
+                        allowed = True
+
+                    alt = dist[u] + self.G[u][v][0][weight]
+                    if allowed and alt < dist[v]:
+                        dist[v] = alt
+                        prev[v] = u
+                        Q[v] = alt
+
+        elif method == 'A*':
+            s_lon, s_lat = float(self.G.nodes[source]['lon']), float(self.G.nodes[source]['lat'])
+            t_lon, t_lat = float(self.G.nodes[target]['lon']), float(self.G.nodes[target]['lat'])
+
+            dist[source] = 0 + self.geod.inv(s_lon, s_lat, t_lon, t_lat)[2]
+
+            Q = heapdict()
+            for v in self.G.nodes:
+                Q[v] = dist[v]
+
+            while Q:
+                u = Q.popitem()[0]
+
+                if u == target:
+                    break
+
+                u_is_switch = True if self.G.nodes[u]['tags'].get('railway') == 'switch' else False
+
+                for v in set(self.G.adj[u]).intersection(set(Q.keys())):  # Neighbors of u that are still in Q
+                    v_lon, v_lat = float(self.G.nodes[v]['lon']), float(self.G.nodes[v]['lat'])
+
+                    if u_is_switch:
+                        u_prev = prev[u]
+                        allowed = True if (u_prev, u, v) in self.G.nodes[u]['attributes'].get('allowed_transits',
+                                                                                         []) else False
+                    else:
+                        allowed = True
+
+                    alt = dist[u] + self.G[u][v][0][weight] + self.geod.inv(v_lon, v_lat, t_lon, t_lat)[2]
+                    if allowed and alt < dist[v]:
+                        dist[v] = alt
+                        prev[v] = u
+                        Q[v] = alt
+        else:
+            raise ValueError("Method not supported")
+
+        S = []  # Shortest path sequence
+        u = target
+
+        if prev[u] or u == source:
+            while u:
+                S.append(u)
+                u = prev[u]
+
+        S.reverse()
+
+        return dist, prev, S
+
     def search_osm_result(self, way_ids: List[int], railway_type="tram"):
         ways = []
 
